﻿/* ETBSaveAsOld.jsx ------ UTF-8 -----------------------------------------------------------------
                 Save FM document or book as old version
   Documentation www.daube.ch/docu/fmaker73.html
   Comment       To be placed in Startup folder of user area
   Reference     Based on SaveBookAsOldVersion 2013-09-04 by Carmen Publishing Inc. (www.frameexpert.com)
                 Traduction française avec l'aide de Babylon et Deepl par Klaus Daube
   Attention     Code must be UTF-8. With Windows cp no umlauts etc. are allowed in menu items.
   Copyleft      2017 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
   History       v1.0  2016-05-24 Klaus Daube, Docu+Design Daube, klaus@daube.ch
                 v1.1  2016-06-03 Save as MIF added. 
                       It is not possible to save to another MIF version as the currnt FM
                       or to MIF-7 (FV_SaveFmtInterchange70) 
                       Only Constants.FV_SaveFmtInterchange is valid in our case
                 v1.2  2017-04-25 Adapted for FM-14 and integration into ETB
                 v1.3  2017-06-30 Revert to dialogue, Inform user about current document location (for ordinary save)
                 v1.4  2018-01-16 New command names
*/ ; // ===========================================================================================
#target framemaker  

var gsETBsavO_title     = { en: "Save book or document in older version",
                            de: "Buch oder dokument in älterer version speichern",
                            fr: "Enregistrer le livre ou document en version plus ancienne"};
var gsETBsavO_NoDoc     = { en: "No book or document active",
                            de: "Kein buch oder dokument aktiv",
                            fr: "Aucun livre ou document actif"};
var gsETBsavO_Target    = { en: "Target folder is required.",
                            de: "Zielverzeichnis is notwendig",
                            fr: "Dossier cible est requise"};
var gsETBsavO_NoTarget  = { en: "Target folder does not exist",
                            de: "Zielverzeichnis nicht vorhanden",
                            fr: "Le dossier cible n'existe pas"};
var gsETBsavO_SelTarget = { en: "Please select a target folder",
                            de: "Bitte zielverzeichnis auswählen",
                            fr: "Veuillez sélectionner un dossier ciblee"};
var gsETBsavO_UnknownFM = { en: "Current FM version is not 10 ... 14",
                            de: "Aktuelle FM version ist nicht 10 ... 14",
                            fr: "Version FM actuelle n'est pas 10 ... 14"};
var gsETBsavO_Success   = { en: "Document or book saved\n\nAttention: Be aware that the current directory is still the Target Folder!",
                            de: "Dokument oder buch gespeichert\n\nAchtung: Das aktuelle verzeichnis ist immer noch das Zielverzeichnis!",
                            fr: "Document ou livre enregistré\n\nAttention : Notez que le répertoire courant est toujours le dossier cible !"};

var goETBsavO_ActiveBook, goETBsavO_ActiveDoc;  
var gsETBsavO_FMversion;                          // FM-8 ... FM-14
var goETBsavO_Folder;                             // target folder
var gbETBsavO_MIF;                                // true for handling MIF
var giETBsavO_FileType;                           // Constants.FV_SaveFmtBinary... | ...FV_SaveFmtInterchange

SwitchUILanguage ();
// DialogSaveOld();                               // un-comment for debugging
SetUpMenu();                                      // out-comment for debugging

function DialogSaveOld() { // =====================================================================
  if (!app.ActiveBook.ObjectValid() && !app.ActiveDoc.ObjectValid()) {
    alert ("ETBSaveAsOld: " + localize (gsETBsavO_NoDoc));
    return;
  }
  var stVersion = { en: "FM target version",
                    de: "FM zielversion",
                    fr: "FM version cible"};
  var stFolder  = { en: "Target folder",
                    de: "Zielverzeichnis",
                    fr: "Dossier cible"};
  var stAdvice  = { en: "Enter folder name or use Browse button",
                    de: "Verzeichnis eingeben oder schaltfläche Blättern benutzen",
                    fr: "Entrez le nom du dossier ou utilisez le bouton Feuilleter"};
  var sbBrowse  = { en: "Browse",
                    de: "Browse",
                    fr: "Feuilleter"};
  var sbSaveFM  = { en: "Save as FM",
                    de: "Als FM sichern",
                    fr: "Enrgistrer comme FM"};
  var sbSaveMIF = { en: "Save as MIF",
                    de: "Als MIF sichern",
                    fr: "Enrgistrer comme  MIF"};
  var sbHelp    = { en: "Help",
                    de: "Hilfe (en)",
                    fr: "Aide (en)"};
  var sbCancel  = { en: "Cancel",
                    de: "Abbrechen",
                    fr: "Annuller"};
  var sbClose   = { en: "OK, close",
                    de: "OK, schliessen",
                    fr: "OK, conclure"};

  wSaveOld = new Window('dialog', localize(gsETBsavO_title), undefined);
	wSaveOld.g1 = wSaveOld.add('group',undefined);
  	wSaveOld.g1.st1 = wSaveOld.g1.add('statictext',undefined,localize(stVersion));
  	wSaveOld.g1.rb = wSaveOld.g1.add('group',undefined);
    	wSaveOld.g1.rb.rb8  = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-8");
    	wSaveOld.g1.rb.rb9  = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-9");
    	wSaveOld.g1.rb.rb10 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-10");
    	wSaveOld.g1.rb.rb11 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-11");
    	wSaveOld.g1.rb.rb12 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-12");
    	wSaveOld.g1.rb.rb13 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-13");
    	wSaveOld.g1.rb.rb14 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-14");
	wSaveOld.g2 = wSaveOld.add('group',undefined);
  	wSaveOld.g2.st2        = wSaveOld.g2.add('statictext',undefined,localize(stFolder));
  	wSaveOld.g2.folderName = wSaveOld.g2.add('edittext',undefined,localize(stAdvice));
  	wSaveOld.g2.folderName.preferredSize.width = 310;;
  	wSaveOld.g2.bBrowse    = wSaveOld.g2.add('button',undefined,localize(sbBrowse));
	wSaveOld.g3 = wSaveOld.add('group',undefined);
  	wSaveOld.g3.alignChildren = ['left',' '];
  	wSaveOld.g3.bSaveFM  = wSaveOld.g3.add('button',undefined,localize(sbSaveFM));
  	wSaveOld.g3.bSaveMIF = wSaveOld.g3.add('button',undefined,localize(sbSaveMIF));
  	wSaveOld.g3.bHelp    = wSaveOld.g3.add('button',undefined,localize(sbHelp));
	wSaveOld.g4 = wSaveOld.add('group',undefined);
  	wSaveOld.g4.alignChildren = ['left',' '];
  	wSaveOld.g4.st3        = wSaveOld.g4.add('statictext',undefined,"___",{multiline:true});
  	wSaveOld.g4.st3.preferredSize = {width:480, height:40};// at least two lines
	wSaveOld.g5 = wSaveOld.add('group',undefined);
  	wSaveOld.g5.alignChildren = ['left',' '];
  	wSaveOld.g5.Close    = wSaveOld.g5.add('button',undefined,localize(sbClose));
  	wSaveOld.g5.Cancel   = wSaveOld.g5.add('button',undefined,localize(sbCancel));
// --- set the radio buttons
  var appVersion = app.VersionMajor;
  var rbuttons   = wSaveOld.g1.rb;
  for (var j = 0; j < rbuttons.children.length; j++) {
    var txt = rbuttons.children[j].text;
    rbuttons.children[j].value = false; 
    rbuttons.children[j].visible = true; 
    if (j+8 == appVersion ) {
      rbuttons.children[j].value = true; 
    }
    if (j+8 > appVersion ) {
      rbuttons.children[j].visible = false; 
    }
  }
// --- behaviour of edit field 
  wSaveOld.g2.folderName.onChange = function () {
    gsFolder = wSaveOld.g2.folderName.text;
 }
// --- behaviour of the buttons
  wSaveOld.g2.bBrowse.onClick  = ButtonBrowse;
  wSaveOld.g3.bSaveFM.onClick  = ButtonSaveFM;
  wSaveOld.g3.bSaveMIF.onClick = ButtonSaveMIF;
  wSaveOld.g3.bHelp.onClick    = ButtonHelp;
  wSaveOld.g5.Close.onClick    = function () {
    wSaveOld.close();
  }
  wSaveOld.g5.Cancel.onClick   = function () {
    wSaveOld.close();
  }
// --- final action in dialgue
  UpdateStatus ("-----");
	wSaveOld.g5.Close.enabled = false;
	wSaveOld.g5.Cancel.enabled = true;
  wSaveOld.show();
} // --- end DialogSaveOld

function ButtonBrowse () { //======================================================================
  var userFolder = new Folder(app.OpenDir);
  var targetFolder = userFolder.selectDlg(localize(gsETBsavO_SelTarget));
  if (targetFolder !== null) {
    wSaveOld.g2.folderName.text = targetFolder.fsName;
  }
} // --- end ButtonBrowse

function ButtonSaveFM () { //======================================================================
//$.bp(true);
  gbETBsavO_MIF = false;
  ButtonSave ();
} // --- end ButtonSaveFM

function ButtonSaveMIF () { //=====================================================================
//$.bp(true);
  gbETBsavO_MIF = true;
  ButtonSave ();
} // --- end ButtonSaveMIF

function ButtonSave () { // =======================================================================
  var folderText = wSaveOld.g2.folderName.text;
  var lastBSlash, fileType;
    if (folderText === "") {                      // Validate the target folder.
    alert (localize(gsETBsavO_Target));
    return;
  } else {                                        // Make sure the folder exists.
    if (Folder(folderText).exists === false) {
      alert (localize(gsETBsavO_NoTarget));
      return;
    } else {
      lastBSlash = folderText.lastIndexOf("\\");
      if (lastBSlash == folderText.length-1) {    // remove terminating \
        folderText = folderText.substring(0, lastBSlash);
      }
      wSaveOld.g2.folderName.text = folderText;
      goETBsavO_Folder = folderText;                      // target folder
    }
  }
  gsETBsavO_FMversion = SelectedRadioButton (wSaveOld.g1.rb); // text from selected radio button
  giETBsavO_FileType = GetFileType (gsETBsavO_FMversion);
// --- for a book
  if (app.ActiveBook.ObjectValid()) {
    goETBsavO_ActiveBook = app.ActiveBook;
    ProcessBook (goETBsavO_ActiveBook.Name);
  } 
// --- for single document
  if (app.ActiveDoc.ObjectValid()) {
    goETBsavO_ActiveDoc = app.ActiveDoc;
    ProcessDoc (goETBsavO_ActiveDoc.Name);
  }
  UpdateStatus (localize(gsETBsavO_Success));
} // --- end ButtonSave

function ButtonHelp () {
//$.bp(true);
  OpenWebURL ("http://www.daube.ch/docu/fmaker73.html");
} // --- end ButtonHelp

function SelectedRadioButton (rbuttons) { //=======================================================
  for (var j = 0; j < rbuttons.children.length; j++) {
    if (rbuttons.children[j].value == true){
      return rbuttons.children[j].text;
    }
  }
} // --- end SelectedRadioButton
function SetUpMenu() { // =========================================================================
// This function MUST NOT be named DefineMenu - because this is a reserved function!
// Errors in ESTK: Menu position can not be changed - see bug report FRMAKER-2752
// Try to solve via menu configuration in ETB
  var sDocMenu     = { en: "Save document as old version …",
                       de: "Dokument in älterer version speichern …",
                       fr: "Enregistrer document comme version ancienne …"};
  var sDocShortLbl = { en: "ESC f s o",
                       de: "ESC d s o",
                       fr: "ESC f s o"};
  var sDocShortCut = { en: "\\!fso",
                       de: "\\!dso",
                       fr: "\\!fso"};
                    
  var sBookMenu     = { en: "Save book as old version",
                        de: "Buch in älterer version speichern …",
                        fr: "Enregistrer livre comme version ancienne …"};
  var sBookShortLbl = { en: "ESC f s o",
                        de: "ESC d s o",
                        fr: "ESC f s o"};
  var sBookShortCut = { en: "\\!fso",
                        de: "\\!dso",
                        fr: "\\!fso"};
                                                // --- Menu definition for documents
  var menuLocation = app.GetNamedMenu("FileMenu");
  var cmdDoc = menuLocation.DefineAndAddCommand(1, "ETB_docSaveOld", localize(sDocMenu), localize(sDocShortCut)); 
  cmdDoc.KeyboardShortcutLabel = localize(sDocShortLbl);  
//cmdDoc.Description = "Save document as old FM version [ETB]";
                                                  // --- Menu definition for books
  menuLocation = app.GetNamedMenu("BookFileMenu"); 
  var cmdBook = menuLocation.DefineAndAddCommand(1, "ETB_bookSaveOld", localize(sBookMenu), localize(sBookShortCut));   
  cmdBook.KeyboardShortcutLabel = localize(sBookShortLbl);  
//cmdBook.Description = "Save book as old FM version [ETB]";
  UpdateMenus();
  
} // --- end SetUpMenu

function Command(cmd) { // ========================================================================
	switch(cmd)  {	                                // Respond to menu command.
		case 1: 
      DialogSaveOld();
      break;
    }
} // --- end Command

function UpdateStatus (sStatus) { // ==============================================================
  wSaveOld.g5.Close.enabled = true;
  wSaveOld.g5.Cancel.enabled = false;
  wSaveOld.g4.st3.text = sStatus;
} // --- end UpdateStatus

function ProcessDoc (name) { // ===================================================================
  var oDoc;
  oDoc = GetDocOrBook (name);                        // Open the document.
  if (oDoc.obj) {
    SaveAsFileType (oDoc.obj, goETBsavO_Folder, giETBsavO_FileType); // Save and close the document.
    if (oDoc.isOpen === false) {
      oDoc.obj.Close (true);
    }
  }
} // --- end ProcessDoc

function ProcessBook (name) { // ==================================================================
  var book = GetDocOrBook (name);                 // Open the book.
  var bookComp = book.obj.FirstComponentInBook;
  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      ProcessDoc (bookComp.Name);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
// update the book (component paths); save the book and close it.
  UpdateComponentPaths (book.obj, goETBsavO_Folder, giETBsavO_FileType);
  SaveAsFileType (book.obj, goETBsavO_Folder, giETBsavO_FileType);
  if (book.isOpen === false) {
    book.obj.Close (true);
  }
} // --- end ProcessBook

function ProcessComponents (book) { // ============================================================
  var bookComp = book.FirstComponentInBook;
  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      ProcessDoc (bookComp.Name);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
} // --- end ProcessComponents

function GetFileType (fmVersion) { // =============================================================
  if (gbETBsavO_MIF) {                                    
        giETBsavO_FileType =  Constants.FV_SaveFmtInterchange;
  } else {
    switch (fmVersion) { 
      case "FM-8":
        giETBsavO_FileType =  Constants.FV_SaveFmtBinary80; break;
      case "FM-9":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary90; break;
      case "FM-10":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary100; break;
      case "FM-11":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary110; break;
      case "FM-12":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary120; break;
      case "FM-13":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary130; break;
      case "FM-14":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary140; break;
      default:
        giETBsavO_FileType = GetDefaultFileType();
    }
  }
  return giETBsavO_FileType;                              // return also in function
} // --- end GetFileType

function GetDefaultFileType () { // ===============================================================
  var appVersion = app.VersionMajor; 
  if (gbETBsavO_MIF) {                                    
        return Constants.FV_SaveFmtInterchange;
  } else {
    switch (appVersion) { 
      case "10":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary100; break;
      case "11":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary110; break;
      case "12":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary120; break;
      case "13":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary130; break;
      case "14":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary140; break;
      default:
        alert (localize(gsETBsavO_UnknownFM));
        exit;
    }
  }
} // --- end GetDefaultFileType

function OpenDocOrBook (filename) { // ============================================================
  var i = 0, doc = 0;
  var docOrBook = {obj: 0, isOpen: false};        // Make an object to return.
  var openProps = GetOpenDefaultParams();         // Get default property list for opening documents or books.
  var retParm = new PropVals();                   // Get a property list to return any error messages.
                                                  // Set specific open property values to open the document or book.
  i=GetPropIndex(openProps,Constants.FS_AlertUserAboutFailure);
  openProps[i].propVal.ival=false;
  i=GetPropIndex(openProps,Constants.FS_MakeVisible);
  openProps[i].propVal.ival=false;
  i=GetPropIndex(openProps,Constants.FS_FileIsOldVersion);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FileIsInUse);
  openProps[i].propVal.ival=Constants.FV_ResetLockAndContinue;
  i=GetPropIndex(openProps,Constants.FS_FontChangedMetric);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FontNotFoundInCatalog);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FontNotFoundInDoc);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_RefFileNotFound);
  openProps[i].propVal.ival=Constants.FV_AllowAllRefFilesUnFindable;

  doc = Open(filename,openProps,retParm);         // Attempt to open the document or book.
  if (doc.ObjectValid() === 1) {
      docOrBook.obj = doc;
  } else {                                        // If the document can't be open, print the errors to the Console.
      PrintOpenStatus(retParm);
  }
  return docOrBook;                               // Return the document or book object.
} // --- end OpenDocOrBook

function GetDocOrBook (filename) { // =============================================================
  if (/\.fm$/i.test (filename) === true) {        // See if the document or book is already open.
    var docOrBook = DocIsOpen (filename);         // Doc
  } else {                                        // Book
    var docOrBook = BookIsOpen (filename);
  }

  if (docOrBook.obj) {
    return docOrBook;
  } else {                                        // The document is not already open, so open it.
    return OpenDocOrBook (filename);
  }
} // --- end GetDocOrBook

function DocIsOpen (filename) { // ================================================================
  var file = File(filename);                      // Make a File object from the file name.
  var fileName = file.fullName.toUpperCase();     // Uppercase the filename for easy comparison.
  var doc = {obj: 0, isOpen: false};              // Make an object to return.
  var currDoc = app.FirstOpenDoc;                 // Loop through the open documents in the session.
  while (currDoc.id) {              // Compare the document’s name with the one we are looking for.
    if (File(currDoc.Name).fullName.toUpperCase() === fileName) { // The document we are looking for is open.
      doc.obj = currDoc;                          // Add it to the object and return it.
      doc.isOpen = true;
      return doc;
    }
    currDoc = currDoc.NextOpenDocInSession;
  }
  return doc;                                     // Document is not open; return the “empty” object.
} // --- end DocIsOpen

function BookIsOpen (filename) { // ===============================================================
  var file = File(filename);                      // Make a File object from the file name.
  var fileName = file.fullName.toUpperCase();     // Uppercase the filename for easy comparison.
  var book = {obj: 0, isOpen: false};             // Make an object to return.
  var currBook = app.FirstOpenBook;               // Loop through the open books in the session.

  while (currBook.id) {             // Compare the document’s name with the one we are looking for.
    if (File(currBook.Name).fullName.toUpperCase() === fileName) { // The document we are looking for is open.
      book.obj = currBook;                        // Add it to the object and return it.
      book.isOpen = true;
      return book;
    }
    currBook = currBook.NextOpenBookInSession;
  }
  return book;                                    // Book is not open; return the “empty” object.
} // --- end BookIsOpen

function UpdateComponentPaths (book, folder, fileType) { // =======================================
  var bookComp = book.FirstComponentInBook;

  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      bookComp.Name = folder +  /\\[^\\]+$/.exec (bookComp.Name)[0];
//  } else if (bookComp.BookComponentFileType === Constants.FV_BK_BOOK) {
//    bookComp.Name = folder +  /\\[^\\]+$/.exec (bookComp.Name)[0];
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
  SaveAsFileType (book, folder, fileType);
} // --- end UpdateComponentPaths

function SaveAsFileType (docOrBook, folder, fileType) { // ========================================
  var params, returnParams, k, fullName, baseName, lastBSlash, lastDot, savePath, extension;
  
  params = GetSaveDefaultParams();
  returnParams = new PropVals();
  k = GetPropIndex (params, Constants.FS_FileType);
  params[k].propVal.ival = fileType;
  fullName = docOrBook.Name;
  lastDot    = fullName.lastIndexOf( "." ) ;      // .fm or .book
  lastBSlash = fullName.lastIndexOf("\\");
  baseName = fullName.substr(lastBSlash+1, lastDot-lastBSlash-1);
  extension  = fullName.substr(lastDot);          // .book or .fm
  if (gbETBsavO_MIF) {extension = ".mif";}
  savePath = folder + "\\" + baseName + extension;
  docOrBook.Save (savePath, params, returnParams);
//$.bp(true);
} // --- end SaveAsFileType

function OpenWebURL (URL) { // ====================================================================
  var path = GetScriptPath () + "\\";
// Create an executable (batch, shortcut) file and execute it
  var tempFile = File (path + "WebShortCut.url");
  tempFile.open("w");
  tempFile.encoding = "UTF-8";
  tempFile.writeln ("[{000214A0-0000-0000-C000-000000000046}]");
  tempFile.writeln ("Prop3=19,2");
  tempFile.writeln ("[InternetShortcut]");
  tempFile.writeln ("URL=" + URL);
  tempFile.close();
  tempFile.execute ();
  tempFile.delete;
}

function GetScriptPath () { //=====================================================================
// Returns path in the function
  var scrPath, winPath, lastBSlash;

// $.fileName: script path (including the file-name of the script)
// platform independent format e.g. /e/_DDDprojects/FM+EN-escript/escript
  scrPath = $.fileName;
  winPath = new File (scrPath);
  scrPath = winPath.fsName;                       // get Windows format
  lastBSlash = scrPath.lastIndexOf("\\");         // \ needs escaping
  scrPath = scrPath.substring(0, lastBSlash);
  return scrPath;                                 // no final \ !
} // --- end GetScriptPath

function SwitchUILanguage () {; // === Switch language according to application language ============
  switch (app.Language) {
    case Constants.FV_LANG_BRITISH:
    case Constants.FV_LANG_ENGLISH: {
      $.locale = "en"; break; 
    }
    case Constants.FV_LANG_CANADIAN_FRENCH: 
    case Constants.FV_LANG_FRENCH: {
      $.locale = "fr"; break; 
    }
    case Constants.FV_LANG_GERMAN: 
    case Constants.FV_LANG_NEW_GERMAN: 
    case Constants.FV_LANG_NEW_SWISS_GERMAN:	
    case Constants.FV_LANG_SWISS_GERMAN:	{
      $.locale = "de"; break; 
    }
    default: {
      $.locale = "en"; break; 
    }
  }
} // --- end SwitchUILanguage
